import os
import json
import re
from tweet_manager.lib import json2csv, format_csv


def create_task_database_structure(
        output_dir: str)\
        -> str:
    """
    Generate the following directory tree: a top dir that will contain
    all the tweet collections if it didn't exist yet and within it the top
    directory for this task with a new and empty metadata file
    """

    # Create the root directory for the tweet collection
    (output_dir, db_name) = os.path.split(output_dir)
    if not os.path.isdir(output_dir):
        print(
            "Building directory to contain the collected tweets at: "
            + os.path.abspath(output_dir)
        )
        os.mkdir(output_dir)
    collection_path = os.path.join(output_dir, db_name)
    if not os.path.isdir(collection_path):
        print("Initializing collection " + db_name + "...")
        os.mkdir(collection_path)
        metadata_path = os.path.join(collection_path, ".metadata.json")
        generate_metadata_file(metadata_path)

    return collection_path


def generate_metadata_file(metadata_path) -> None:
        print("Executing generate_metadata_file")
        file_metadata = {}  # type: Dict
        metadata = {}
        metadata["files"] = file_metadata

        with open(metadata_path, "w") as f:
            json.dump(metadata, f)


def add_newfile_to_metadata(file_path: str, metadata_path: str) -> None:
    """
    Add a new dictionary structure to the metadata file that contains
    information about a newly added CSV. This should just be user for files
    that have just been added to the collection, because it initializes the
    count to 0
    """

    print("Executing add_newfile_to_metadata")
    try:
        with open(metadata_path, "r+") as f:
            metadata_file = json.load(f)
            metadata_file["files"][file_path] = {}
            metadata_file["files"][file_path]["count"] = 0
            f.seek(0)
            f.truncate()
            json.dump(metadata_file, f)
    except IOError:
        generate_metadata_file(metadata_path)
        add_newfile_to_metadata(file_path, metadata_path)


def increase_metadata_count(
        metadata_path: str,
        file_path: str,
        increase: int = 1)\
        -> None:
    """
    Use this when one tweet is appended to one of the CSVs in the
    collection. This function will update the metadata file by increasing
    by x the corresponding dictionary structure
    """

    print("Executing increase_metadata_count")
    try:
        with open(metadata_path, "r+") as f:
            metadata_file = json.load(f)
            metadata_file["files"][file_path]["count"] += increase
            f.seek(0)
            f.truncate()
            json.dump(metadata_file, f)
    except IOError:
        generate_metadata_file(metadata_path)
        increase_metadata_count(metadata_path, file_path, increase)


def create_tweet_output_path(
        header: str,
        tweet: dict,
        output_dir: str)\
        -> str:
    collection_path = create_task_database_structure(output_dir)

    # Extract year, month and date from the tweet using a regex
    matchObj = re.search(
        r"^(\d{4})-(\d{2})-(\d{2}) \d{2}:\d{2}:\d{2}$",
        str(tweet["created_at"]))
    year = matchObj.group(1)
    month = matchObj.group(2)
    day = matchObj.group(3)
    date = (year, month, "")

    # Classify the tweet chronologically
    tweet_output_path = json2csv.mkdir_tweet_date(date, collection_path)
    tweet_output_file = os.path.join(tweet_output_path, day + ".csv")

    # If the CSV file didn't already exist, initialize it with a header
    if os.path.isfile(tweet_output_file) is False:
        with open(tweet_output_file, "w") as fw:
            fw.write(header.strip())
        add_newfile_to_metadata(
            tweet_output_file,
            os.path.join(collection_path, ".metadata.json"))

    return tweet_output_file


def convert_tweet_to_csv(header: str, tweet: dict) -> str:
    # Flatten the tweet and store it in status_flat
    status_flat = json2csv.flatten_dictionary(tweet)

    # Convert the flat JSON to CSV format
    # 1st arg: flat tweet, 2nd arg: activate array compression, 3rd arg:
    # number of array compression levels, 4th arg: remove dollars mode
    status_csv = json2csv.json2csv(status_flat, True, 5, False)

    csv_appendable_line = format_csv.get_csv_line(header, status_csv)

    return csv_appendable_line
